#ifndef _MOBJECTS_H_
#define _MOBJECTS_H_

// GLU kniznica
#include <GL/glu.h>

#include "m3Dmath.h"

typedef struct _Ssphere { // pologula namiesto skutocnej oblohy
 GLfloat *v_array;        // pole vertex coords
 GLfloat *t_array;        // pole texture coords
};

typedef struct _Sclouds { // zakryvena plocha namiesto oblakov
 GLfloat x,y,z;           // vrchol
 GLfloat a;               // priesvitnost
 GLfloat u,v;             // suradnice textury
};

// trieda premennych a funkcii pre pracu s jednoduchymi OpenGL objektmi
class mObjects {
public:
 mObjects();          // konstruktor
 virtual ~mObjects(); // destruktor

 void obj_axis(GLfloat color);     // deklaracia objektov - suradnicova sustava
 void obj_XZplane(GLfloat color);  // deklaracia objektov - XZ sachovnica
 void obj_XZplane1(GLfloat color); // deklaracia objektov - XZ fill sachovnica
 void obj_XZplane2(GLfloat size);  // definicia objektov - XZ texturovana plocha
 void obj_XYZplane(GLfloat size);  // XYZ plocha (zvysene okraje)
 void obj_cube0(GLfloat size);     // deklaracia objektov - kocka biela
 void obj_cube1(GLfloat size);     // deklaracia objektov - kocka farebna
 void obj_cube2(GLfloat size);     // deklaracia objektov - kocka ftexturovana
 // deklaracia objektov - kocka texturovana s normalou v kazdom vrchole
 void obj_cube2N(GLfloat size);

 void obj_quad0(GLfloat Xsize,GLfloat Ysize);

 // inicializacia kvadratickych objektov
 int obj_quadratic_init(void);
 // vykreslenie kvadratickeho objektu
 void obj_quadratic(GLuint type,GLfloat param,GLfloat color);

 int obj_skybox_init(char *mydir); // inicializacia skyboxu
 void obj_skybox_draw(void);       // vykreslenie skyboxu

 int obj_sky_init(char *mydir);   // inicializacia oblohy
 void obj_sky_draw(GLfloat tadd); // vykreslenie oblohy

 // inicializacia sosovkoveho efektu
 int obj_lensflare_init(char *mydir);
 // sosovkovy efekt pohladu do svetla (slnka)
 void obj_lensflare(GLfloat c1,GLfloat c2,GLfloat c3,
                    GLfloat t1,GLfloat t2,GLfloat t3,
                    GLfloat u1,GLfloat u2,GLfloat u3,
                    GLfloat r1,GLfloat r2,GLfloat r3,
                    GLfloat l1,GLfloat l2,GLfloat l3);
private:
 GLfloat t_a,t_b;     // pomocne premenne
 GLuint t_c,t_d;      // pomocne premenne
 GLfloat y1,y2,y3,y4; // pomocne premenne

 GLuint skybox_tex_id[6]; // identifikatory textur skybox
 GLuint skybox_DL_id;     // identifikator displaylist skybox

 GLuint sky_tex_id[2]; // identifikatory textur oblohy
 GLuint sky_DL_id;     // identifikator displaylist oblohy

 GLuint lensflare_tex_id[6]; // identifikatory textur lensflare

 GLUquadricObj *obj_quad; // kvadratik objekt

 // premenne pre vypocet sosovkoveho efektu
 m3Dvector CameraP,TargetP,UpV,RightV,LightP;
 m3Dvector CameraLightVector,LightIntersectVector;
 float l;
 m3Dvector IntersectP;
 m3Dvector middleP,p[4];

 GLint VP[4];                        // viewport
 GLdouble MVmatrix[16],PRmatrix[16]; // transformacne matice
 GLdouble winx,winy,winz;            // vysledne suradnice
 GLdouble flareZ; // hlbka svetla
 GLfloat bufferZ; // hlbka ulozena v zasobniku hlbky
 
 bool isOccluded(m3Dvector point);
 void lflare(GLuint tex_id,
             GLfloat r,GLfloat g,GLfloat b,GLfloat a,
             m3Dvector point,
             GLfloat scale);
			 
 GLfloat sky_time; // pre zmenu polohy oblakov
 
 _Ssphere Ssphere; // pologula predstavujuca oblohu
 _Sclouds Sclouds; // zakrivena plocha predstavujuca oblaky
 
 int create_Ssphere(GLfloat r,GLint t1,GLint t2); // vytvorenie pologule
 void draw_Ssphere(GLint t1,GLint t2);            // vykreslenie pologule
 void destroy_Ssphere(void);                      // uvolnenie pamate

 _Sclouds *PlaneVertices; // zoznam vrcholov zakrivenej plochy
 GLuint NumPlaneVertices; // pocet vrcholov zakrivenej plochy
 GLuint *Indices;         // zoznam odkazov na konkretny vrchol
 GLuint NumIndices;       // pocet odkazov na konkretny vrchol

 int create_Sclouds(GLfloat r1,GLfloat r2,GLfloat alphaP); // vytvorenie zakrivenej plochy
 void draw_Sclouds(void);                                  // vykreslenie zakrivenej plochy
 void destroy_Sclouds(void);                               // uvolnenie pamate
};

#endif
